/*
 * omg-scansipdestinations
 *
 * Copyright (c) 2014 Johannes Noe
 * Licensed under the MIT license.
 */

'use strict';

var logger = require("omg-logger");
var _ = require("lodash");
var exec = require('child_process').exec;


function customFilter(object, result) {
    if (object.hasOwnProperty('string_sipDefaultGatewayIpAddress') && object.string_sipDefaultGatewayIpAddress !== "") {
        result.push({
            address: object.string_sipDefaultGatewayIpAddress,
            proto: object.string_sipDefaultGatewayIpMode.toLowerCase(),
            port: object.string_sipDefaultGatewayIpPort,
            info: object.string_Description,
            disabled: object.string_disabled,
            resolved: null
        });
    }
    for (var i = 0; i < Object.keys(object).length; i++) {
        if (typeof object[Object.keys(object)[i]] === "object") {
            if (object[Object.keys(object)[i]].bool_CfgValid !== false) {
                customFilter(object[Object.keys(object)[i]], result);
            }
        }
    }
}

exports.start = function(startCallback, outputCallback) { //this function has a name!
    logger.info("scan starting");
    startCallback();

    exec("./config2json.lua", {
        cwd: __dirname + "/lua/"
    }, function(error, stdout) { // , stderr
        if (error) {
            logger.error(error.message);
            startCallback(error);
            return;
        }
        var config;
        var data = {};
        var res = [],
            unique;
        try {
            config = JSON.parse(stdout);
        } catch (e) {
            logger.error("\n\n\n");
            logger.error("no /data/config.lua ?\n\n\n");
        }

        customFilter(config, res);

        unique = _.uniq(res, function(rule) { // , key , res
            return rule.address + rule.proto + rule.port + rule.disabled + rule.info;
        });

        _.each(unique, function(rule) {
            var propertyName = (rule.address + rule.proto + rule.port + rule.disabled + rule.info);
            data[propertyName] = rule;
        });
        outputCallback(null, data);

        _.each(data, function(host, key) {
            exec("./omgprobe.lua " + host.address + " " + host.port + " " + host.proto + " --json", {
                cwd: __dirname + "/lua/"
            }, function(error, stdout) { //, stderr
                stdout = JSON.parse(stdout);
                data[key].resolved = stdout.resolved;
                outputCallback(null, data);
            });
        });
    });
};